#!/usr/bin/perl
#
# Firewall Tester client v.0.3
# by Andrea Barisani (lcars@infis.univ.trieste.it) 
# Copyright (c) 2001 / Code under GPL (see COPYING)

require 'getopts.pl';
use Net::RawIP;
use Net::PcapUtils;
use NetPacket::Ethernet qw(:strip);
use NetPacket::IP qw(:strip);
use NetPacket::TCP;
use NetPacket::UDP;
use NetPacket::ICMP;
Getopts('f:d:k:s:v');

die "Usage $0 -f <conf_file> -d <delay, 0.25 = 250 ms> -k <cksum value, es. 60000> -s <sleep time, 1 = 1 s> -v" unless ($opt_f & $opt_d);

my $delay = $opt_d;
my $tcp_payload = 'ftestertcpprobe';
my $udp_payload = 'ftesterudpprobe';
my $icmp_payload = 'ftestericmpprobe';
my $seqprobe = 'seqprobe';
my $stopsignal = 'ftesterstop';
my $verbose = $opt_v;
my $checksum = $opt_k;

my $count = 0;
my @flags_table;

open (OUT,">>./ftest.log") || die "cannot create log file";
print OUT ("# Ftest started on ".`date`);

for ($count = 0; $count < 64; $count++) {
    $flags_table[$count] = "$flags_table[$count]P" if $count & 8;
    $flags_table[$count] = "$flags_table[$count]U" if $count & 32;
    $flags_table[$count] = "$flags_table[$count]S" if $count & 2;
    $flags_table[$count] = "$flags_table[$count]A" if $count & 16;
    $flags_table[$count] = "$flags_table[$count]F" if $count & 1;
    $flags_table[$count] = "$flags_table[$count]R" if $count & 4;
}

open (CONF,$opt_f) or die "Can't open $opt_f";
while (<CONF>) {
	next if  /^#|^\s/; 
	my @fields = split /:/;
	die unless @fields >= 6;
	chomp @fields;	
	
	$psh = 0; $ack = 0; $rst = 0; $syn = 0; $urg = 0; $fin = 0;
	
	$psh = 1 if $fields[4] =~ /P/;
	$ack = 1 if $fields[4] =~ /A/;
	$rst = 1 if $fields[4] =~ /R/;
	$syn = 1 if $fields[4] =~ /S/;
	$urg = 1 if $fields[4] =~ /U/;
	$fin = 1 if $fields[4] =~ /F/;
		
	if ($_ !~ /-|stop_signal|connect/) {
		if ($fields[5] =~ TCP) {
			print($fields[0],":",$fields[1]," > ",$fields[2],":",$fields[3]," ",$fields[4]," ","TCP"," ",$fields[6],"\n") if $verbose;
			send_tcp($fields[0],$fields[1],$fields[2],$fields[3],$fields[4],$tcp_payload,$fields[6]);
		}
		if ($fields[5] =~ UDP) {
			print($fields[0],":",$fields[1]," > ",$fields[2],":",$fields[3]," ","UDP"," ",$fields[6],"\n") if $verbose;
			send_udp($fields[0],$fields[1],$fields[2],$fields[3],$fields[4],$udp_payload,$fields[6]);
		}
		if ($fields[5] =~ ICMP) {
			print($fields[0]," > ",$fields[2]," ","ICMP"," ",$fields[6]," ",$fields[7],"\n") if $verbose;
			send_icmp($fields[0],$fields[1],$fields[2],$fields[3],$fields[4],$icmp_payload,$fields[6],$fields[7]);
		}
	} else {

		if ($fields[0] =~ /[0-9]+\-[0-9]+/ ) {
			my @ip_range = split (/\.|-/,$fields[0]);
		
			for my $i ( $ip_range[3] .. $ip_range[4] ) {
				$source_addr = $ip_range[0].'.'.$ip_range[1].'.'.$ip_range[2].'.'.$i;
			
				if ($fields[1] =~ /[0-9]+\-[0-9]+/ ) {
					@src_port_range = split (/-/,$fields[1]);
					for my $l ( $src_port_range[0] .. $src_port_range[1] ) {
					
					if ($fields[3] =~ /[0-9]+\-[0-9]+/ ) {
						@dst_port_range = split (/-/,$fields[3]);
						for my $h ( $dst_port_range[0] .. $dst_port_range[1] ) {
							
							if ($fields[5] =~ TCP) {
								print($source_addr,":",$l," > ",$fields[2],":",$h," ",$fields[4]," ","TCP"," ",$fields[6],"\n") if $verbose;
								send_tcp($source_addr,$l,$fields[2],$h,$fields[4],$tcp_payload,$fields[6]);
							}
							if ($fields[5] =~ UDP) {
								print($source_addr,":",$l," > ",$fields[2],":",$h," ","UDP"," ",$fields[6],"\n") if $verbose;
								send_udp($source_addr,$l,$fields[2],$h,$fields[4],$udp_payload,$fields[6]);
							}
							if ($fields[5] =~ ICMP) {
								print($source_addr," > ",$fields[2]," ","ICMP"," ",$fields[6]," ",$fields[7],"\n") if $verbose;
								send_icmp($source_addr,$l,$fields[2],$h,$fields[4],$icmp_payload,$fields[6],$fields[7]);
							}
						}
							
					} else { 
						print ("@fields\n");
						die "/* one or more entries in ftest.conf are incorrect! */" }
					}
			
				} else { 
					print ("@fields\n");
					die "/* one or more entries in ftest.conf are incorrect! */" }	
			} 
		
		} else {
			
			if ($fields[0] =~ /connect/ ) {
				my @connect_fields = split /=|:/;
				chomp @connect_fields;			
	
				send_tcp_connect($connect_fields[1],$connect_fields[2],$connect_fields[3],$connect_fields[4],"S",$seqprobe,$connect_fields[7]);
				print("Sent Syn Probe => ",$connect_fields[1],":",$connect_fields[2]," > ",$connect_fields[3],":",$connect_fields[4]," ","S"," ","TCP","\n") if $verbose;
				
				print("Sleeping for $opt_s seconds\n");
				sleep ($opt_s);
				
				send_tcp_connect($connect_fields[1],$connect_fields[2],$connect_fields[3],$connect_fields[4],"A","ackprobe",$connect_fields[7],9,66);
				print("Sent Ack Reply => ",$connect_fields[1],":",$connect_fields[2]," > ",$connect_fields[3],":",$connect_fields[4]," ","A"," ","TCP","\n") if $verbose;

				send_tcp($connect_fields[1],$connect_fields[2],$connect_fields[3],$connect_fields[4],$connect_fields[5],$tcp_payload,$connect_fields[7],9,66);
				print($connect_fields[1],":",$connect_fields[2]," > ",$connect_fields[3],":",$connect_fields[4]," ",$connect_fields[5]," ","TCP"," ",$connect_fields[7],"\n") if $verbose;
	
		} else {	
			
			if ($fields[0] =~ /stop_signal/ ) {
				my @stop_fields = split /=|:/;
				chomp @stop_fields;
	
				if ($stop_fields[6] =~ TCP) {
					print("Stop packet => ",$stop_fields[1],":",$stop_fields[2]," > ",$stop_fields[3],":",$stop_fields[4]," ",$stop_fields[5]," ","TCP","\n") if $verbose;
	   			 	send_tcp($stop_fields[1],$stop_fields[2],$stop_fields[3],$stop_fields[4],$stop_fields[5],$stopsignal,$stop_fields[7]);
					print OUT ("# Ftest stopped on ".`date`);
					close (CONF);
				}
				if ($stop_fields[6] =~ UDP) {
					print("Stop packet => ",$stop_fields[1],":",$stop_fields[2]," > ",$stop_fields[3],":",$stop_fields[4]," ","UDP","\n") if $verbose;
	   			 	send_udp($stop_fields[1],$stop_fields[2],$stop_fields[3],$stop_fields[4],$stop_fields[5],$stopsignal,$stop_fields[7]);
					print OUT ("# Ftest stopped on ".`date`);
					close (CONF);
				}
				if ($stop_fields[6] =~ ICMP) {
					print("Stop packet => ",$source_fields[1]," > ",$stop_fields[3]," ","ICMP"," ",$fields[7]," ",$fields[8],"\n") if $verbose;
					send_icmp($stop_fields[1],$stop_fields[2],$stop_fields[3],$stop_fields[4],$stop_fields[5],$stopsignal,$stop_fields[6],$stop_fields[7]);
					print OUT ("# Ftest stopped on ".`date`);
					close (CONF);
				}
			} else { 
				print ("@fields\n");
				die  "/* one or more entries in ftest.conf are incorrect! */" }	
			}
		}
	}
}

sub send_tcp {
	
		$a = new Net::RawIP({
		ip => {
			ttl => 200,
			id => 1,
			saddr => $_[0],
                	daddr => $_[2],
			tos => $_[6],
			},
		tcp => {
			check => $checksum,	
                	source => $_[1],
                	dest => $_[3],
			psh => $psh, 
			ack => $ack,
			rst => $rst,
			syn => $syn, 
			urg => $urg, 
			fin => $fin, 
                	seq => $_[7],
			ack_seq => $_[8], 
			data => $_[5]}
		});
		$a->send($delay,1);

		print OUT ("$_[0]:$_[1] > $_[2]:$_[3] ");
		
		$flags = 0;	
		
		if ($_[4] =~ /P/ ) {
			$flags = 8;
			}
               	if ($_[4] =~ /A/ ) {
			$flags = $flags + 16;
			}
               	if ($_[4] =~ /R/ ) {
			$flags = $flags + 4;
			}
               	if ($_[4] =~ /S/ ) {
			$flags = $flags + 2;
			}
               	if ($_[4] =~ /U/ ) {
			$flags = $flags + 32;
			}
               	if ($_[4] =~ /F/ ) {
			$flags = $flags + 1;
			}
       		print OUT ("$flags_table[$flags] TCP $_[6]\n"); 
	}

sub send_tcp_connect {
       
       		$real_syn = $syn; $real_ack = $ack;
       		
	       	if ($_[4] =~ /S/ ) {
			$syn = 1;
			}
               	if ($_[4] =~ /A/ ) {
			$ack = 1;
			}
		
		$a = new Net::RawIP({
		ip => {
			ttl => 200,
			id => 1,
			saddr => $_[0],
                	daddr => $_[2],
			tos => $_[6],
			},
		tcp => {
			check => $checksum,	
                	source => $_[1],
                	dest => $_[3],
			ack => $ack,
			syn => $syn, 
                	seq => $_[7],
			ack_seq => $_[8], 
			data => $_[5]}
		});
		$a->send($delay,1);
		
		$syn = $real_syn; $ack = $real_ack;
	
	}

sub send_udp {
	
		$a = new Net::RawIP({
		ip => {
			ttl => 200,
			id => 1,
			saddr => $_[0],
                	daddr => $_[2],
			tos => $_[6],
			},
		udp => {
			check => $checksum,	
                	source => $_[1],
                	dest => $_[3],
                	data => $_[5]}
		});
		$a->send($delay,1);

		print OUT ("$_[0]:$_[1] > $_[2]:$_[3] UDP $_[6]\n");
		
	}

sub send_icmp {
	
		$a = new Net::RawIP({
		ip => {
			ttl => 200,
			id => 1,
			saddr => $_[0],
                	daddr => $_[2],
			},
		icmp => {
			check => $checksum,	
                	type => $_[6],
			code => $_[7],
			data => $_[5]}
		});
		$a->send($delay,1);

		print OUT ("$_[0] > $_[2] ICMP $_[6] $_[7]\n");
	}
