#!/usr/bin/perl
#
# Firewall Tester listener daemon v.0.3
# by Andrea Barisani (lcars@infis.univ.trieste.it)
# Copyright (c) 2001 / Code under GPL (see COPYING)

require 'getopts.pl';
use Net::RawIP;
use Net::PcapUtils;
use NetPacket::Ethernet qw(:strip);
use NetPacket::IP qw(:strip);
use NetPacket::TCP;
use NetPacket::UDP;
use NetPacket::ICMP;
Getopts('c:v');

my $verbose = $opt_v;
my $connect_ack = 'connect_ack';
my @ttls = split (/:/,$opt_c);

system "echo $ttls[0] > /proc/sys/net/ipv4/ip_default_ttl" if $opt_c;

print "listening...\n";

open (OUT,">>./ftestd.log") || die "cannot create log file";
print OUT ("# Ftestd started on ".`date`);

my $count = 0;
my @flags_table;

for ($count = 0; $count < 64; $count++) {
    $flags_table[$count] = "$flags_table[$count]P" if $count & 8;
    $flags_table[$count] = "$flags_table[$count]U" if $count & 32;
    $flags_table[$count] = "$flags_table[$count]S" if $count & 2;
    $flags_table[$count] = "$flags_table[$count]A" if $count & 16;
    $flags_table[$count] = "$flags_table[$count]F" if $count & 1;
    $flags_table[$count] = "$flags_table[$count]R" if $count & 4;
}

sub process_pkt {
	my ($arg, $hdr, $pkt) = @_;
	my $ip_obj = NetPacket::IP->decode(eth_strip($pkt));
        my $tcp_obj = NetPacket::TCP->decode(ip_strip(eth_strip($pkt)));
        my $udp_obj = NetPacket::UDP->decode(ip_strip(eth_strip($pkt)));
        my $icmp_obj = NetPacket::ICMP->decode(ip_strip(eth_strip($pkt)));
	
	if ($tcp_obj->{data} =~ /ftestertcpprobe/ ) {
			open (OUT,">>./ftestd.log");
			print ("$ip_obj->{src_ip}:$tcp_obj->{src_port} > $ip_obj->{dest_ip}:$tcp_obj->{dest_port} $flags_table[$tcp_obj->{flags}] TCP $ip_obj->{tos}\n") if $verbose;
			print OUT ("$ip_obj->{src_ip}:$tcp_obj->{src_port} > $ip_obj->{dest_ip}:$tcp_obj->{dest_port} $flags_table[$tcp_obj->{flags}] TCP $ip_obj->{tos}\n");
		}
		
	if ($udp_obj->{data} =~ /ftesterudpprobe/ ) {
			open (OUT,">>./ftestd.log");
			print ("$ip_obj->{src_ip}:$udp_obj->{src_port} > $ip_obj->{dest_ip}:$udp_obj->{dest_port} UDP $ip_obj->{tos}\n") if $verbose;
			print OUT ("$ip_obj->{src_ip}:$udp_obj->{src_port} > $ip_obj->{dest_ip}:$udp_obj->{dest_port} UDP $ip_obj->{tos}\n");
		}
			
	if ($icmp_obj->{data} =~ /ftestericmpprobe/ ) {
			open (OUT,">>./ftestd.log");
			print ("$ip_obj->{src_ip} > $ip_obj->{dest_ip} ICMP $icmp_obj->{type} $icmp_obj->{code}\n") if $verbose;
			print OUT ("$ip_obj->{src_ip} > $ip_obj->{dest_ip} ICMP $icmp_obj->{type} $icmp_obj->{code}\n");
		}
	
	if ($tcp_obj->{data} =~ /seqprobe/ ) {
			print ("Received Syn Probe: $ip_obj->{src_ip}:$tcp_obj->{src_port} > $ip_obj->{dest_ip}:$tcp_obj->{dest_port} $flags_table[$tcp_obj->{flags}] TCP\n") if $verbose;
				
			$a = new Net::RawIP({
			ip => {
				ttl => $ttls[1],
				id => 1,
				saddr => $ip_obj->{dest_ip},
                		daddr => $ip_obj->{src_ip},
				},
			tcp => {
                		source => $tcp_obj->{dest_port},
                		dest => $tcp_obj->{src_port},
				syn => 1, 
                		ack => 1,
				seq => '66',
				ack_seq => '9', 
				}
			});
			$a->send($delay,1);
			print ("Sent Syn Ack Reply: $ip_obj->{dest_ip}:$tcp_obj->{dest_port} > $ip_obj->{src_ip}:$tcp_obj->{src_port} SA TCP\n") if $verbose;
		}	
	
	if ($tcp_obj->{data} =~ /ftesterstop/ ) {
			print ("Stop packet => $ip_obj->{src_ip}:$tcp_obj->{src_port} > $ip_obj->{dest_ip}:$tcp_obj->{dest_port} $flags_table[$tcp_obj->{flags}] TCP\n") if $verbose;
			print OUT ("$ip_obj->{src_ip}:$tcp_obj->{src_port} > $ip_obj->{dest_ip}:$tcp_obj->{dest_port} $flags_table[$tcp_obj->{flags}] TCP $ip_obj->{tos}\n");
			print OUT ("# Ftestd finished on ".`date`);
			close(OUT);
			system "echo 64 > /proc/sys/net/ipv4/ip_default_ttl" if $opt_c;
			die "received disconnection signal";
		}
	}

Net::PcapUtils::loop(\&process_pkt, FILTER => 'ip');

